package com.xydl.cac.iec104;

import com.xydl.cac.exception.BusinessException;
import com.xydl.cac.model.StaticVariable;
import com.xydl.cac.service.Iec104PointService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;

@Service
@Slf4j
public class Iec104ServerService {

    @Resource
    Iec104PointService iec104PointService;

    @PostConstruct
    private void init() {
        try {
            iec104PointService.findAll();
            if (!CollectionUtils.isEmpty(StaticVariable.point104_Cache)) {
                this.startServer(2404);
            }
        } catch (Exception ignore) {
        }
    }

    public void startServer(int port) throws BusinessException {
        try {
            Iec104Server server = new Iec104Server();
            server.start(port);
            StaticVariable.iec104Server = server;
        } catch (Exception e) {
            log.error("启动IEC104服务端异常.", e);
            throw new BusinessException(e.getMessage());
        }
    }

    public void stopServer() {
        this.doStopServer();
    }

    private void doStopServer() {
        if (StaticVariable.iec104Server != null) {
            StaticVariable.iec104Server.stop();
            StaticVariable.iec104Server = null;
        }
    }

    @PreDestroy
    private void stop() {
        StaticVariable.shutdown = 1;
        this.doStopServer();
    }

}
