package com.xydl.cac.task;

import com.xydl.cac.entity.Iec104Point;
import com.xydl.cac.entity.WarnRule;
import com.xydl.cac.model.StaticVariable;
import com.xydl.cac.service.Iec104PointService;
import com.xydl.cac.service.WarnRuleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
@Slf4j
public class CacheTask {

    @Resource
    WarnRuleService ruleService;
    @Resource
    Iec104PointService iec104PointService;

    @Scheduled(cron = "0 0 1 * * ?")
    public void clearCache1() {
        StaticVariable.unit_Cache.clear();
        StaticVariable.modevType_Cache = null;
        StaticVariable.jg_Cache = null;
        StaticVariable.zsb_Cache = null;
        StaticVariable.rule_Cache.clear();
        StaticVariable.ruleRelationMap.clear();
        StaticVariable.paramRelationMap.clear();
    }

    @Scheduled(cron = "0 30 9 * * ?")
    public void clearCache9() {
        StaticVariable.doneWarnMap.clear();
    }

    @Scheduled(initialDelay = 30 * 1000, fixedDelay = 60 * 1000)
    public void refreshRule() {
        try {
            List<WarnRule> list = ruleService.listAll(null);
            for (WarnRule item : list) {
                WarnRule rule = StaticVariable.rule_Cache.get(item.getId());
                if (rule == null) {
                    item.initComparator(item.getComparator(), item.getOperator());
                    StaticVariable.rule_Cache.put(item.getId(), item);
                }
                if (item.getNSensor() != null && item.getTypePoint() != null) {
                    String key = item.getNSensor().getDevId() + "_" + item.getTypePoint().getField();
                    StaticVariable.ruleRelationMap.put(key, item.getId());
                }
            }
        } catch (Exception e) {
            log.error("CacheTask.refreshRule error.", e);
        }
    }

    @Scheduled(initialDelay = 5 * 60 * 1000, fixedDelay = 5 * 60 * 1000)
    public void refresh104() {
        iec104PointService.findAll();
    }
}
